module tvsignal(clk,modebutton,invbutton,horpulse,blue,red,green);

input clk,invbutton,modebutton;
output horpulse;
output blue,red,green;
reg horpulse;
//reg blue,red,green;

reg [9:0] cntrow;				// input clock - 16 MHz
reg [9:0] cntframe;
wire cntframe624;
wire row2,cntrow3ff,bcnt111;
reg horblnk,vertblnk;
wire beghorblnk,endhorblnk;
wire begvertblnk,endvertblnk;
wire blue1,red1,green1;
wire beghorsync,endhorsync0,endhorsync1,endhorsync2;
reg [6:0] bcnt;
reg [2:0] barcnt;
reg [1:0] lenhorsync,lenhorsync1;
reg [5:0] columncnt;
reg [4:0] rowgridcnt;
reg column,row;
wire grid,centerpoint,chess;
reg columntrig,rowtrig;
wire [1:0] graybars,gridmode,chessboard;
reg b,g,r;
reg mbprocessed;
reg [1:0] pict;
reg invprocessed;
reg inv;


//begin and end of horizontal synchronization pulse
assign beghorsync=cntrow[0] & cntrow[1] & cntrow[2] & cntrow[3] & cntrow[4] & cntrow[5] & cntrow[6] & cntrow[7] & cntrow[8];
assign endhorsync0=cntrow[1]&cntrow[3]&cntrow[6];		//length of horsync intime of visible part of frame - 4.7us (75 tacts) (74)
assign endhorsync1=cntrow[0]&cntrow[2]&cntrow[5];		//length of equalizing pulse - 2.35 us (38 tacts) (37)
assign endhorsync2=cntrow[2]&cntrow[4]&cntrow[5]&cntrow[7]&cntrow[8];		//length of framesync - 27.3 us (437 tacts) (436)

always @(negedge clk)
begin
	cntrow<=cntrow+1;				//counter for making rows - divider of input 16 MHz on 2^10
	case (horpulse)
       0:
			case (lenhorsync1)
				0:if (endhorsync0)		horpulse<=1;
				1:if (endhorsync1)		horpulse<=1;
				2:if (endhorsync2)		horpulse<=1;
			endcase
       1:
       begin
			if (beghorsync)
				if (lenhorsync==0)
				begin
					if (cntrow[9])
						horpulse<=0;
				end
				else horpulse<=0;					
       end
	endcase
end

//counter for making frames - divider of double row frequency on 625
assign row2=cntrow[8]&cntrow[7]&cntrow[6]&cntrow[5]&cntrow[4]&cntrow[3]&cntrow[2]&cntrow[1]&cntrow[0]; //dummy variable

assign cntframe624=cntframe[4]&cntframe[5]&cntframe[6]&cntframe[9];		//  1001110000H=624D

always @(negedge clk)
if (row2)
begin
	case (lenhorsync)
		0:if (cntframe[1]&cntframe[3]&cntframe[5]&cntframe[6]&cntframe[9])
				lenhorsync<=1;//after 618
		1:if (cntframe[0]&cntframe[1]&cntframe[2]&cntframe[3]&cntframe[5]&cntframe[6]&cntframe[9])
				lenhorsync<=2; //after 623 (begin of frame, begin of vertsync)
			else if (cntframe[3]&(!cntframe[9]))
				lenhorsync<=0;	//after 8 (begin of active part of frame)
		2:if (cntframe[0]&cntframe[1])
				lenhorsync<=1;	//after 3 (end of vertsync)
		default:
			lenhorsync<=2;
	endcase
	
	lenhorsync1<=lenhorsync;

	if (cntframe624)
	begin
		cntframe<=0;
	end
	else
		cntframe<=cntframe+1;
end

//making horizontal blank indicating work interval of row
//begin and end of blank in binary code (in us)
assign beghorblnk=cntrow[0] & cntrow[1] & cntrow[2] & cntrow[5] & cntrow[7] & (!cntrow[9]);	// 10.5 us (168 tacts) (167)
assign endhorblnk=cntrow[0] & cntrow[1] & cntrow[2] & cntrow[5] & cntrow[6] & cntrow[7] & cntrow[8] & cntrow[9]; // 62.5 us (1000 tacts) (99)

always @(negedge clk)
	case (horblnk)
       0:
       begin
           if (beghorblnk==1)
				horblnk=1;
       end
       1:
       begin
           if (endhorblnk==1)
				horblnk=0;
       end
	endcase

//making vertical blank indicating work interval of frame
//begin and end of blank in binary code (in row/2)
assign begvertblnk=cntframe[0] & cntframe[2] & cntframe[3] & cntframe[5] & (!cntframe[9]) & cntrow[0] & cntrow[1] & cntrow[2] & cntrow[5] & cntrow[7]; 
assign endvertblnk=cntframe[0] & cntframe[1] & cntframe[3] & cntframe[5] & cntframe[6] & cntframe[9] & cntrow[0] & cntrow[1] & cntrow[2] & cntrow[5] & cntrow[6] & cntrow[7] & cntrow[8];// & cntrow[9];

always @(negedge clk)
	case (vertblnk)
       0:
       begin
           if (begvertblnk==1)
				vertblnk=1;
       end
       1:
       begin
           if (endvertblnk==1)
				vertblnk=0;
       end
	endcase

//counter for color bars (length of each bar = 7 us, instead of right 6.4 us)
assign cntrow3ff=cntrow[9] & row2;//dummy variable
assign bcnt111=bcnt[0] & bcnt[1] & bcnt[3] & bcnt[5] & bcnt[6]; //7us/62.5ns=112 (temporary 107)

always @(negedge clk)
if (cntrow3ff | bcnt111)				
	bcnt<=0;
else
	bcnt<=bcnt+1;

//making blue, red, and green signals
always @(negedge clk)
if ((!bcnt[0])&(!bcnt[1])&(!bcnt[2])&(bcnt[3])&(!bcnt[4])&(bcnt[5])&(!bcnt[6]))
	if (!horpulse)
		barcnt<=(-1);
	else
		barcnt<=barcnt+1;

assign blue1=!barcnt[0], red1=!barcnt[1], green1=!barcnt[2];

//making grid and chessboard
assign centerpoint=(cntrow==584 | cntrow==585) & (cntframe==332 | cntframe==333);
assign grid=column | row | centerpoint;
always @(negedge clk)
begin
	if (columncnt==55 | cntrow==51)
	begin
		columncnt<=0;
		column<=1;								//every 55-th pixel is column line
//		columntrig<=!columntrig;				//every 55-th pixel is inverse of square of the chessboard
	end
	else
	begin
		columncnt<=columncnt+1;
		column<=0;
	end

	if (cntrow3ff)
		if (rowgridcnt==23 | cntframe==19 | cntframe==18)
		begin
			rowgridcnt<=0;							//after 12-th line reset of rowgridcount
			row<=1;									//every 48-th line is horizontal line
		end
		else
		begin
			rowgridcnt<=rowgridcnt+1;
			row<=0;
		end
end

assign chess=columntrig^rowtrig;
always @(negedge clk)
begin
	if (cntrow==51)
		columntrig<=0;							//reset columntrig after 51 tact from line beginning
	else
		if (columncnt==0)
			columntrig<=!columntrig;			//every 55-th pixel is inverse of square of the chessboard

	if (cntrow3ff)
	begin
		if (cntframe==19 | cntframe==18)
			rowtrig<=0;							//reset of rowtrig after 12-th line
		else
		if (rowgridcnt==0)
				rowtrig<=!rowtrig;				//every 48-th line is inverse of square in the chessboard
	end
end

//switching pictures
//always pict=0;
always @(negedge clk)
if (cntframe624 & cntrow3ff)
	case (modebutton)
	0:
		if (!mbprocessed)
		begin
			mbprocessed<=1;
			if (pict[1])
					pict<=0;
				else
					pict<=pict+1;
		end
	1:	mbprocessed<=0;
	endcase

//inversing picture
//assign inv=!invbutton;
always @(negedge clk)
if (cntframe624 & cntrow3ff)
	case (invbutton)
	0:
		if (!invprocessed)
		begin
			invprocessed<=1;
			inv<=!inv;
		end
	1:	invprocessed<=0;
	endcase

assign graybars=0,gridmode=1,chessboard=2;
always
begin
	case (pict)
		graybars:		begin	b=blue1;	r=red1;		g=green1;	end
		gridmode:		begin	b=grid;		r=grid;		g=grid;		end
		chessboard:		begin	b=chess;	r=chess;	g=chess;	end
		default:		begin	b=1;	r=1;		g=1;	end
	endcase
end

assign blue=(inv^b) & horblnk & vertblnk, red=(inv^r) & horblnk & vertblnk, green=(inv^g) & horblnk & vertblnk;

endmodule
